<?php

namespace EWR\Discord\Repository;

use XF\Mvc\Entity\AbstractCollection;
use XF\Mvc\Entity\Repository;

class Log extends Repository
{
	public function writeLog($user, $type, $new, $old = [])
	{
		$log = \XF::em()->create('EWR\Discord:Log');
		$log->bulkSet([
			'user_id' => $user->user_id,
			'log_type' => $type,
			'log_old' => $old,
			'log_new' => $new,
		]);
		$log->save();
	}

	public function findLogs()
	{
		return $this->finder('EWR\Discord:Log')
			->setDefaultOrder('log_date', 'DESC');
	}
	
	public function groupLogs($logs)
	{
		$grouped = [];
		$roles = [];
		
		$discordRepo = \XF::repository('EWR\Discord:Discord');
		$discordRepo->assertSetup();
		
		foreach ($discordRepo->getGuild('/roles') AS $role)
		{
			$roles[$role['id']] = $role['name'].' ('.$role['id'].')';
		}

		foreach ($logs AS $key => $log)
		{
			if ($log->log_type == 'x2d_roles')
			{
				$olds = [];
				$news = [];
				
				foreach (explode(',', $log->log_old) AS $role)
				{
					if (!empty($roles[$role]))
					{
						$olds[$role] = $roles[$role];
					}
				}
				$log->log_old = implode('<br/>',$olds);
				
				foreach (explode(',', $log->log_new) AS $role)
				{
					if (!empty($roles[$role]))
					{
						$news[$role] = $roles[$role];
					}
				}
				$log->log_new = implode('<br/>',$news);
			}
			
			$groupKey = "{$log->user_id}-{$log->log_date}";
			if (!isset($grouped[$groupKey]))
			{
				$grouped[$groupKey] = [
					'user' => \XF::em()->find('XF:User', $log->user_id),
					'date' => $log->log_date,
					'type' => $log->log_type,
					'logs' => [],
					'changes' => [],
				];
			}

			$grouped[$groupKey]['logs'][$key] = $log;
		}

		return $grouped;
	}

	public function pruneLogs($cutOff = null)
	{
		if ($cutOff === null)
		{
			$length = $this->options()->EWRdiscord_logLength;
			if (!$length)
			{
				return 0;
			}

			$cutOff = \XF::$time - 86400 * $length;
		}

		return $this->db()->delete('ewr_discord_logs', 'log_date < ?', $cutOff);
	}
}