<?php

namespace EWR\Discord;

use XF\Db\Schema\Alter;
use XF\Db\Schema\Create;

use XF\AddOn\AbstractSetup;
use XF\AddOn\StepRunnerInstallTrait;
use XF\AddOn\StepRunnerUninstallTrait;
use XF\AddOn\StepRunnerUpgradeTrait;

class Setup extends AbstractSetup
{
	use StepRunnerInstallTrait;
	use StepRunnerUpgradeTrait;
	use StepRunnerUninstallTrait;

	public function installStep1()
	{
		$this->schemaManager()->alterTable('xf_node', function(Alter $table)
		{
			$table->addColumn('ewr_discord_options', 'blob')->nullable();
		});
		
		$this->schemaManager()->alterTable('xf_user_option', function(Alter $table)
		{
			$table->addColumn('ewr_discord_options', 'blob')->nullable();
		});
		
		$this->schemaManager()->alterTable('xf_user_connected_account', function(Alter $table)
		{
			$table->addColumn('ewr_discord_syncdate', 'int', 10)->setDefault(0);
		});
	}
	
	public function installStep2()
	{
		$this->db()->insert('xf_connected_account_provider', [
			'provider_id' => 'ewr_discord',
			'provider_class' => 'EWR\Discord:Provider\Discord',
			'display_order' => 100,
			'options' => ''
		], true);
	}
	
	public function installStep3()
	{
		$this->schemaManager()->createTable('ewr_discord_logs', function(Create $table)
		{
			$table->checkExists(true);
			$table->addColumn('user_id',				'int', 10);
			$table->addColumn('log_id',					'int', 10)->autoIncrement();
			$table->addColumn('log_date',				'int', 10);
			$table->addColumn('log_type',				'varchar', 100);
			$table->addColumn('log_old',				'blob');
			$table->addColumn('log_new',				'blob');
			$table->addPrimaryKey('log_id');
		});
	}
	
	public function upgrade2000Step1()
	{
		$this->installStep1();
		$this->installStep2();
	}
	
	public function upgrade2106Step1()
	{
		$this->installStep3();
	}
	
	public function upgrade2110Step1()
	{
		$this->schemaManager()->alterTable('xf_node', function(Alter $table)
		{
			$table->renameColumn('discord_options', 'ewr_discord_options')->setDefault(NULL);
		});
		
		$this->schemaManager()->alterTable('xf_user_option', function(Alter $table)
		{
			$table->renameColumn('discord_options', 'ewr_discord_options')->setDefault(NULL);
		});
		
		$this->schemaManager()->alterTable('xf_user_connected_account', function(Alter $table)
		{
			$table->renameColumn('sync_date', 'ewr_discord_syncdate')->setDefault(0);
		});
	}
	
	public function upgrade2110Step2()
	{
		$nodes = $this->db()->fetchAll("SELECT node_id, ewr_discord_options FROM xf_node");
		
		foreach ($nodes AS $node)
		{
			if ($node['ewr_discord_options'] != NULL)
			{
				$this->db()->update('xf_node',
					[ 'ewr_discord_options' => json_encode(unserialize($node['ewr_discord_options'])) ],
					'node_id = ?', $node['node_id']
				);
			}
		}
	}
	
	public function upgrade2112Step1(array $stepParams)
	{
		$this->db()->update('xf_connected_account_provider',
			['provider_id' => 'ewr_discord'], 'provider_id = ?', 'discord'
		);
	}
	
	public function upgrade2112Step2(array $stepParams)
	{
		$position = empty($stepParams[0]) ? 0 : $stepParams[0];
		$perPage = 250;
	 
		$db = $this->db();
	 
		if (!isset($stepParams['max']))
		{
			$stepParams['max'] = $db->fetchOne("
				SELECT MAX(user_id)
				FROM xf_user_connected_account
				WHERE provider = 'discord'
			");
		}
	 
		$userIds = $db->fetchAllColumn($db->limit("
				SELECT DISTINCT user_id
				FROM xf_user_connected_account
				WHERE user_id > ?
					AND provider = 'discord'
				ORDER BY user_id
			", $perPage
		), $position);
		
		if (!$userIds) { return true; }
	 
		$db->beginTransaction();
	 
		$connectedAccounts = \XF::finder('XF:UserConnectedAccount')->where([
			['user_id', $userIds],
			['provider', 'discord']
		])->with(['User'])->fetch();
		
		$repo = \XF::repository('XF:ConnectedAccount');
		foreach ($connectedAccounts as $connectedAccount)
		{
			$connectedAccount->fastUpdate('provider', 'ewr_discord');
			$repo->rebuildUserConnectedAccountCache($connectedAccount->User);
		}
		
		$db->commit();
	 
		$next = end($userIds);
		return [$next, "{$next} / {$stepParams['max']}", $stepParams];
	}
	
	public function uninstallStep1()
	{
		$this->schemaManager()->alterTable('xf_node', function(Alter $table)
		{
			$table->dropColumns('ewr_discord_options');
		});
		
		$this->schemaManager()->alterTable('xf_user_option', function(Alter $table)
		{
			$table->dropColumns('ewr_discord_options');
		});
		
		$this->schemaManager()->alterTable('xf_user_connected_account', function(Alter $table)
		{
			$table->dropColumns('ewr_discord_syncdate');
		});
		
		$this->schemaManager()->dropTable('ewr_discord_logs');
	}
	
	public function uninstallStep2()
	{
		$this->db()->delete('xf_connected_account_provider', 'provider_id = ?', 'ewr_discord');
		$this->db()->delete('xf_user_connected_account', 'provider = ?', 'ewr_discord');
	}
}